# Generated by Django 5.2.6 on 2025-09-22 23:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        (
            "whitebox_plugin_flight_management",
            "0003_flightsession_name_flightsessionrecording_status",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="flightsessionrecording",
            name="provided_by_ct",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
        ),
        migrations.AddField(
            model_name="flightsessionrecording",
            name="provided_by_id",
            field=models.IntegerField(null=True),
        ),
    ]
